
    subroutine mass(GP, LP, iLayer)

    use VariableDefination
    implicit NONE
    type(GlobalParameters)    ::  GP
    type(LayerParameters)     ::  LP(100)
    integer*4  ::  iLayer

    if(GP%CoordinatesType.eq.1) then
        call massCartesian(GP, LP, iLayer)
    elseif(GP%CoordinatesType.eq.0) then
        call massSpherical(GP, LP, iLayer)
    endif


    end subroutine mass



    subroutine momentum(GP, LP, iLayer)

    use VariableDefination
    implicit NONE
    type(GlobalParameters)    ::  GP
    type(LayerParameters)     ::  LP(100)
    integer*4  ::  iLayer

    call reconstructFlowDepth(GP, LP, iLayer)
    if(GP%CoordinatesType.eq.1) then
        if(LP(iLayer)%GoverningEquationType.eq.1) then
            call momentumLinearCartesian(GP, LP, iLayer)
        elseif(LP(iLayer)%GoverningEquationType.eq.0) then
            call momentumNonlinearCartesian(GP, LP, iLayer)
        endif
    elseif(GP%CoordinatesType.eq.0) then
        if(LP(iLayer)%GoverningEquationType.eq.1) then
            call momentumLinearSpherical(GP, LP, iLayer)
        elseif(LP(iLayer)%GoverningEquationType.eq.0) then
            call momentumNonlinearSpherical(GP, LP, iLayer)
        endif
    endif

    end subroutine momentum



    subroutine massCartesian(GP, LP, iLayer)

    use VariableDefination
    implicit NONE
    type(GlobalParameters)    ::  GP
    type(LayerParameters)     ::  LP(100)
    integer*4  ::  iLayer, irank
    integer*4  ::  nstartx, nendx, nstarty, nendy
    integer*4  ::  i, j, k
    real*8     ::  m1, m2, n1, n2
    real*8     ::  CPUTime1, CPUTime2

    irank = GP%irank
    call CPU_TIME(CPUTime1);

    if(irank.lt.LP(iLayer)%nsize) then

    nstartx = LP(iLayer)%PartitionInfo(irank+1,1)
    nendx   = LP(iLayer)%PartitionInfo(irank+1,2)
    nstarty = LP(iLayer)%PartitionInfo(irank+1,3)
    nendy   = LP(iLayer)%PartitionInfo(irank+1,4)
    if(LP(iLayer)%Level.gt.1) then !do not calculate GHOST grids on child layers
        nstartx = MAX(nstartx,GP%nGHOST+1)
        nendx   = MIN(nendx,LP(iLayer)%NX-GP%nGHOST)
        nstarty = MAX(nstarty,GP%nGHOST+1)
        nendy   = MIN(nendy,LP(iLayer)%NY-GP%nGHOST)
    endif

    !//// solve the mass equation ////!
    do j = nstarty, nendy
    do i = nstartx, nendx
        !/// exclude permanent dry cells from equations ///! 
        if(LP(iLayer)%Z(i,j).le.-GP%PermanentDryLimit) then
            LP(iLayer)%H(2,i,j) = 0.0; cycle
        endif
        if(i.eq.1) then
            m1 = 0.0d0
        else
            m1 = LP(iLayer)%M(1,i-1,j)
        endif
        if(i.eq.LP(iLayer)%NX) then
            m2 = 0.0d0
        else
            m2 = LP(iLayer)%M(1,i,j)
        endif
        if(j.eq.1) then
            n1 = 0.0d0
        else
            n1 = LP(iLayer)%N(1,i,j-1)
        endif
        if(j.eq.LP(iLayer)%NY) then
            n2 = 0.0d0
        else
            n2 = LP(iLayer)%N(1,i,j)
        endif
        LP(iLayer)%H(2,i,j) = LP(iLayer)%H(1,i,j) - &
            LP(iLayer)%CC1*(m2-m1) - LP(iLayer)%CC2*(n2-n1)
    enddo
    enddo
 
    !//// calculate inundation in flooding zone ///!
    do k = 1,LP(iLayer)%FloodingCellsCount
        i = LP(iLayer)%FloodingCells(k,1)
        j = LP(iLayer)%FloodingCells(k,2)

        !/// wet -> dry ///!
        if(LP(iLayer)%Z(i,j)+LP(iLayer)%H(1,i,j).gt.0.0.and.&
          LP(iLayer)%H(2,i,j)-LP(iLayer)%H(1,i,j).lt.0.0.and.&
            LP(iLayer)%Z(i,j)+LP(iLayer)%H(2,i,j).le.GP%MinWaterDepth) then
            if(LP(iLayer)%Z(i,j).gt.0.0) then
                LP(iLayer)%H(2,i,j) = -LP(iLayer)%Z(i,j)
            else
                LP(iLayer)%H(2,i,j) = 0.0
            endif
        endif

        !/// dry -> wet/dry ///
        if(LP(iLayer)%Z(i,j)+LP(iLayer)%H(1,i,j).le.0.0) then
            if(LP(iLayer)%H(2,i,j)-LP(iLayer)%H(1,i,j).gt.0.0) then ! dry -> wet
                LP(iLayer)%H(2,i,j) = LP(iLayer)%H(2,i,j)-LP(iLayer)%H(1,i,j)-LP(iLayer)%Z(i,j)
            else ! dry ->dry
                if(LP(iLayer)%Z(i,j).gt.0.0) then
                    LP(iLayer)%H(2,i,j) = -LP(iLayer)%Z(i,j)
                else
                    LP(iLayer)%H(2,i,j) = 0.0
                endif
            endif
        endif
    enddo

    !/// store maximum water height in this layer ///!
    nstartx = LP(iLayer)%PartitionInfo(irank+1,1)
    nendx   = LP(iLayer)%PartitionInfo(irank+1,2)
    nstarty = LP(iLayer)%PartitionInfo(irank+1,3)
    nendy   = LP(iLayer)%PartitionInfo(irank+1,4)
    do j = nstarty, nendy
    do i = nstartx, nendx
        LP(iLayer)%Hmax(i,j) = MAX(LP(iLayer)%Hmax(i,j), LP(iLayer)%H(2,i,j))
    enddo
    enddo

    endif !if: irank is used

    call CPU_TIME(CPUTime2); GP%CPUTime(irank+1,3) = GP%CPUTime(irank+1,3)+CPUTime2-CPUTime1

    end subroutine massCartesian




    subroutine massSpherical(GP, LP, iLayer)

    use VariableDefination
    implicit NONE
    type(GlobalParameters)    ::  GP
    type(LayerParameters)     ::  LP(100)
    integer*4  ::  iLayer, irank
    integer*4  ::  nstartx, nendx, nstarty, nendy
    integer*4  ::  i, j, k
    real*8     ::  m1, m2, n1, n2, tmp1, tmp2
    real*8     ::  CPUTime1, CPUTime2

    irank = GP%irank
    call CPU_TIME(CPUTime1)

    if(irank.lt.LP(iLayer)%nsize) then

    nstartx = LP(iLayer)%PartitionInfo(irank+1,1)
    nendx   = LP(iLayer)%PartitionInfo(irank+1,2)
    nstarty = LP(iLayer)%PartitionInfo(irank+1,3)
    nendy   = LP(iLayer)%PartitionInfo(irank+1,4)
    if(LP(iLayer)%Level.gt.1) then !do not calculate GHOST grids on child layers
        nstartx = MAX(nstartx,GP%nGHOST+1)
        nendx   = MIN(nendx,LP(iLayer)%NX-GP%nGHOST)
        nstarty = MAX(nstarty,GP%nGHOST+1)
        nendy   = MIN(nendy,LP(iLayer)%NY-GP%nGHOST)
    endif

    !//// solve the mass equation ////!
    do j = nstarty,nendy
    do i = nstartx,nendx
        !/// exclude permanent dry cells from equations ///! 
        if(LP(iLayer)%Z(i,j).le.-GP%PermanentDryLimit) then
            LP(iLayer)%H(2,i,j) = 0.0; cycle
        endif
        if(i.eq.1) then
            m1 = 0.0d0
        else
            m1 = LP(iLayer)%M(1,i-1,j)
        endif
        if(i.eq.LP(iLayer)%NX) then
            m2 = 0.0d0
        else
            m2 = LP(iLayer)%M(1,i,j)
        endif
        if(j.eq.1) then
            n1 = 0.0d0; tmp1 = COS((LP(iLayer)%ymin-0.5*LP(iLayer)%dy)*GP%PI/180.0)
        else
            n1 = LP(iLayer)%N(1,i,j-1); tmp1 = LP(iLayer)%CSY(j-1)
        endif
        if(j.eq.LP(iLayer)%NY) then
            n2 = 0.0d0; tmp2 = COS((LP(iLayer)%ymax+0.5*LP(iLayer)%dy)*GP%PI/180.0)
        else
            n2 = LP(iLayer)%N(1,i,j); tmp2 = LP(iLayer)%CSY(j)
        endif
        LP(iLayer)%H(2,i,j) = LP(iLayer)%H(1,i,j) - LP(iLayer)%CS1(j)*(m2-m1) - &
            LP(iLayer)%CS2(j)*(n2*tmp2-n1*tmp1)
    enddo
    enddo   
     
    !//// calculate inundation in flooding zone ///!
    do k = 1,LP(iLayer)%FloodingCellsCount
        i = LP(iLayer)%FloodingCells(k,1)
        j = LP(iLayer)%FloodingCells(k,2)

        !/// wet -> dry ///!
        if(LP(iLayer)%Z(i,j)+LP(iLayer)%H(1,i,j).gt.0.0.and.&
          LP(iLayer)%H(2,i,j)-LP(iLayer)%H(1,i,j).lt.0.0.and.&
            LP(iLayer)%Z(i,j)+LP(iLayer)%H(2,i,j).le.GP%MinWaterDepth) then
            if(LP(iLayer)%Z(i,j).gt.0.0) then
                LP(iLayer)%H(2,i,j) = -LP(iLayer)%Z(i,j)
            else
                LP(iLayer)%H(2,i,j) = 0.0
            endif
        endif

        !/// dry -> wet/dry ///
        if(LP(iLayer)%Z(i,j)+LP(iLayer)%H(1,i,j).le.0.0) then
            if(LP(iLayer)%H(2,i,j)-LP(iLayer)%H(1,i,j).gt.0.0) then ! dry -> wet
                LP(iLayer)%H(2,i,j) = LP(iLayer)%H(2,i,j)-LP(iLayer)%H(1,i,j)-LP(iLayer)%Z(i,j)
            else ! dry ->dry
                if(LP(iLayer)%Z(i,j).gt.0.0) then
                    LP(iLayer)%H(2,i,j) = -LP(iLayer)%Z(i,j)
                else
                    LP(iLayer)%H(2,i,j) = 0.0
                endif
            endif
        endif
    enddo

    !/// store maximum water height in this layer ///!
    nstartx = LP(iLayer)%PartitionInfo(irank+1,1)
    nendx   = LP(iLayer)%PartitionInfo(irank+1,2)
    nstarty = LP(iLayer)%PartitionInfo(irank+1,3)
    nendy   = LP(iLayer)%PartitionInfo(irank+1,4)
    do j = nstarty, nendy
    do i = nstartx, nendx
        LP(iLayer)%Hmax(i,j) = MAX(LP(iLayer)%Hmax(i,j), LP(iLayer)%H(2,i,j))
    enddo
    enddo

    endif !if: irank is used

    call CPU_TIME(CPUTime2); GP%CPUTime(irank+1,3) = GP%CPUTime(irank+1,3)+CPUTime2-CPUTime1

    end subroutine massSpherical



    subroutine momentumLinearCartesian(GP, LP, iLayer)

    use VariableDefination
    implicit NONE
    type(GlobalParameters)    ::  GP
    type(LayerParameters)     ::  LP(100)
    integer*4   ::  iLayer, irank
    integer*4   ::  nstartx, nendx, nstarty, nendy
    integer*4   ::  istart, iend, jstart, jend
    integer*4   ::  i, j, k, FluxSign
    real*8      ::  z1, z2, h1, h2
    real*8      ::  CPUTime1, CPUTime2

    irank = GP%irank
    call CPU_TIME(CPUTime1)

    if(irank.lt.LP(iLayer)%nsize) then
   
    nstartx = LP(iLayer)%PartitionInfo(irank+1,1)
    nendx   = LP(iLayer)%PartitionInfo(irank+1,2)
    nstarty = LP(iLayer)%PartitionInfo(irank+1,3)
    nendy   = LP(iLayer)%PartitionInfo(irank+1,4)
    
    !///calculate flux component M///!
    if(LP(iLayer)%Level.eq.1) then !calculate all grids in top layer
        istart = MAX(1,nstartx); iend = MIN(LP(iLayer)%NX-1,nendx)
        jstart = MAX(1,nstarty); jend = MIN(LP(iLayer)%NY,nendy)
    elseif(LP(iLayer)%Level.gt.1) then !do not calculate GHOST grids in child layer
        istart = MAX(GP%nGHOST+1,nstartx); iend = MIN(LP(iLayer)%NX-GP%nGHOST-1,nendx)
        jstart = MAX(GP%nGHOST+1,nstarty); jend = MIN(LP(iLayer)%NY-GP%nGHOST,nendy)
    endif
    do j = jstart,jend
    do i = istart,iend
        z1 = LP(iLayer)%Z(i,j);   h1 = LP(iLayer)%H(2,i,j)
        z2 = LP(iLayer)%Z(i+1,j); h2 = LP(iLayer)%H(2,i+1,j)

        !/// exclude the cell where water cannot flow ///!
        call checkFluxDirection(GP%PermanentDryLimit, z1, z2, h1, h2, FluxSign)
        if(FluxSign.eq.999) then
            LP(iLayer)%M(2,i,j) = 0.0d0; cycle
        endif 
        !/// calculate the value of M ///!
        LP(iLayer)%M(2,i,j) = LP(iLayer)%M(1,i,j) - LP(iLayer)%CC3*LP(iLayer)%D_M(i,j)*(h2-h1)
        !/// check if the direction of M is right ///!
        if(FluxSign.ne.0) then
            if(LP(iLayer)%M(2,i,j)*FluxSign.lt.0.0) LP(iLayer)%M(2,i,j) = 0.0d0
        endif
    enddo
    enddo

    !///calculate flux component N///!
    if(LP(iLayer)%Level.eq.1) then !calculate all grids in top layer 
        istart = MAX(1,nstartx); iend = MIN(LP(iLayer)%NX,nendx)
        jstart = MAX(1,nstarty); jend = MIN(LP(iLayer)%NY-1,nendy)
    elseif(LP(iLayer)%Level.gt.1) then !do not calculate GHOST grids on child layers
        istart = MAX(GP%nGHOST+1,nstartx); iend = MIN(LP(iLayer)%NX-GP%nGHOST,nendx)
        jstart = MAX(GP%nGHOST+1,nstarty); jend = MIN(LP(iLayer)%NY-GP%nGHOST-1,nendy)
    endif
    do j = jstart,jend
    do i = istart,iend
        z1 = LP(iLayer)%Z(i,j);   h1 = LP(iLayer)%H(2,i,j)
        z2 = LP(iLayer)%Z(i,j+1); h2 = LP(iLayer)%H(2,i,j+1)
        
        !/// exclude the cell where water cannot flow ///!
        call checkFluxDirection(GP%PermanentDryLimit, z1, z2, h1, h2, FluxSign)
        if(FluxSign.eq.999) then
            LP(iLayer)%N(2,i,j) = 0.0d0; cycle
        endif
        !/// calculate the value of N ///!
        LP(iLayer)%N(2,i,j) = LP(iLayer)%N(1,i,j) - LP(iLayer)%CC4*LP(iLayer)%D_N(i,j)*(h2-h1)
        !/// check if the direction of N is right ///!
        if(FluxSign.ne.0) then
            if(LP(iLayer)%N(2,i,j)*FluxSign.lt.0.0) LP(iLayer)%N(2,i,j) = 0.0
        endif
    enddo
    enddo
      
    !//// set zero water flux around dry cells ////!
    do k = 1,LP(iLayer)%PermanentDryCellsCount
        i = LP(iLayer)%PermanentDryCells(k,1)
        j = LP(iLayer)%PermanentDryCells(k,2)
        if(i.le.LP(iLayer)%NX-1) LP(iLayer)%M(2,i,j) = 0.0d0
        if(i.ge.2) LP(iLayer)%M(2,i-1,j) = 0.0d0
        if(j.le.LP(iLayer)%NY-1) LP(iLayer)%N(2,i,j) = 0.0d0
        if(j.ge.2) LP(iLayer)%N(2,i,j-1) = 0.0d0
    enddo

    endif !if irank is used 
    call CPU_TIME(CPUTime2); GP%CPUTime(irank+1,3) = GP%CPUTime(irank+1,3)+CPUTime2-CPUTime1

    end subroutine momentumLinearCartesian



    subroutine momentumLinearSpherical(GP, LP, iLayer)

    use VariableDefination
    implicit NONE
    type(GlobalParameters)    ::  GP
    type(LayerParameters)     ::  LP(100)
    integer*4   ::  iLayer, irank
    integer*4   ::  nstartx, nendx, nstarty, nendy
    integer*4   ::  istart, iend, jstart, jend
    integer*4   ::  i, j, k, FluxSign
    real*8      ::  z1, z2, h1, h2, flux
    real*8      ::  n1, n2, n3, n4, m1, m2, m3, m4
    real*8      ::  CPUTime1, CPUTime2

    irank = GP%irank
    call CPU_TIME(CPUTime1)

    if(irank.lt.LP(iLayer)%nsize) then

    nstartx = LP(iLayer)%PartitionInfo(irank+1,1)
    nendx   = LP(iLayer)%PartitionInfo(irank+1,2)
    nstarty = LP(iLayer)%PartitionInfo(irank+1,3)
    nendy   = LP(iLayer)%PartitionInfo(irank+1,4)

    !/// calculate flux component M ///!
    if(LP(iLayer)%Level.eq.1) then !calculate all grids on top layer
        istart = MAX(1,nstartx); iend = MIN(LP(iLayer)%NX-1,nendx)
        jstart = MAX(1,nstarty); jend = MIN(LP(iLayer)%NY,nendy)
    elseif(LP(iLayer)%Level.gt.1) then !do not calculate GHOST grids on child layers
        istart = MAX(GP%nGHOST+1,nstartx); iend = MIN(LP(iLayer)%NX-GP%nGHOST-1,nendx)
        jstart = MAX(GP%nGHOST+1,nstarty); jend = MIN(LP(iLayer)%NY-GP%nGHOST,nendy)
    endif
    do j = jstart,jend
    do i = istart,iend
        z1 = LP(iLayer)%Z(i,j);   h1 = LP(iLayer)%H(2,i,j)
        z2 = LP(iLayer)%Z(i+1,j); h2 = LP(iLayer)%H(2,i+1,j)

        !/// exclude the cell where water cannot flow ///!
        call checkFluxDirection(GP%PermanentDryLimit, z1, z2, h1, h2, FluxSign)
        if(FluxSign.eq.999) then !water cannot flow between these two cells
            LP(iLayer)%M(2,i,j) = 0.0d0; cycle
        endif
        !/// calculate the value of M ///!
        LP(iLayer)%M(2,i,j) = LP(iLayer)%M(1,i,j) - LP(iLayer)%CS3(j)*LP(iLayer)%D_M(i,j)*(h2-h1)
        !/// Coriolis force ///!
        if(j.eq.1) then
            n1 = 0.0d0; n2 = 0.0d0
        else
            n1 = LP(iLayer)%N(1,i,j-1); n2 = LP(iLayer)%N(1,i+1,j-1)
        endif
        if(j.eq.LP(iLayer)%NY) then
            n3 = 0.0d0; n4 = 0.0d0
        else
            n3 = LP(iLayer)%N(1,i,j); n4 = LP(iLayer)%N(1,i+1,j)
        endif
        flux = 0.25*(n1+n2+n3+n4)
        LP(iLayer)%M(2,i,j) = LP(iLayer)%M(2,i,j) + LP(iLayer)%CPX(j)*flux

        !/// check if the direction of M is right ///!
        if(FluxSign.ne.0) then
            if(LP(iLayer)%M(2,i,j)*FluxSign.lt.0.0) LP(iLayer)%M(2,i,j) = 0.0d0
        endif
    enddo
    enddo

    !/// calculate flux component N ///!
    if(LP(iLayer)%Level.eq.1) then !calculate all grids in top layer
        istart = MAX(1,nstartx); iend = MIN(LP(iLayer)%NX,nendx)
        jstart = MAX(1,nstarty); jend = MIN(LP(iLayer)%NY-1,nendy)
    elseif(LP(iLayer)%Level.gt.1) then !do not calculate GHOST grids on child layers
        istart = MAX(GP%nGHOST+1,nstartx); iend = MIN(LP(iLayer)%NX-GP%nGHOST,nendx)
        jstart = MAX(GP%nGHOST+1,nstarty); jend = MIN(LP(iLayer)%NY-GP%nGHOST-1,nendy)
    endif
    do j = jstart,jend
    do i = istart,iend
        z1 = LP(iLayer)%Z(i,j);   h1 = LP(iLayer)%H(2,i,j)
        z2 = LP(iLayer)%Z(i,j+1); h2 = LP(iLayer)%H(2,i,j+1)

        !/// exclude the cell where water cannot flow ///!
        call checkFluxDirection(GP%PermanentDryLimit, z1, z2, h1, h2, FluxSign)
        if(FluxSign.eq.999) then !water cannot flow between these two cells
            LP(iLayer)%N(2,i,j) = 0.0d0; cycle
        endif
        !/// calculate the value of N ///!
        LP(iLayer)%N(2,i,j) = LP(iLayer)%N(1,i,j) - LP(iLayer)%CS4*LP(iLayer)%D_N(i,j)*(h2-h1)
        !/// Coriolis force ///!
        if(i.eq.1) then
            m1 = 0.0d0; m2 = 0.0d0
        else
            m1 = LP(iLayer)%M(1,i-1,j); m2 = LP(iLayer)%M(1,i-1,j+1)
        endif
        if(i.eq.LP(iLayer)%NX) then
            m3 = 0.0d0; m4 = 0.0d0
        else
            m3 = LP(iLayer)%M(1,i,j); m4 = LP(iLayer)%M(1,i,j+1)
        endif
        flux = 0.25*(m1+m2+m3+m4)
        LP(iLayer)%N(2,i,j) = LP(iLayer)%N(2,i,j) - LP(iLayer)%CPY(j)*flux

        !/// check if the direction of N is right ///!
        if(FluxSign.ne.0) then
            if(LP(iLayer)%N(2,i,j)*FluxSign.lt.0.0) LP(iLayer)%N(2,i,j) = 0.0d0
        endif
    enddo
    enddo

    !//// set zero water flux at boundaries of dry cells ////!
    do k = 1,LP(iLayer)%PermanentDryCellsCount
        i = LP(iLayer)%PermanentDryCells(k,1)
        j = LP(iLayer)%PermanentDryCells(k,2)
        if(i.le.LP(iLayer)%NX-1) LP(iLayer)%M(2,i,j) = 0.0d0
        if(i.ge.2) LP(iLayer)%M(2,i-1,j) = 0.0d0
        if(j.le.LP(iLayer)%NY-1) LP(iLayer)%N(2,i,j) = 0.0d0
        if(j.ge.2) LP(iLayer)%N(2,i,j-1) = 0.0d0
    enddo

    endif !if irank is used 
    call CPU_TIME(CPUTime2); GP%CPUTime(irank+1,3) = GP%CPUTime(irank+1,3)+CPUTime2-CPUTime1

    end subroutine momentumLinearSpherical



    subroutine reconstructFlowDepth(GP, LP, iLayer)

    use VariableDefination
    implicit NONE
    type(GlobalParameters)    ::  GP
    type(LayerParameters)     ::  LP(100)
    integer*4   ::  iLayer, irank
    integer*4   ::  nstartx, nendx, nstarty, nendy
    integer*4   ::  i, j, s
    real*8      ::  z1, z2, h1, h2, m0, n0
    real*8      ::  CPUTime1, CPUTime2

    irank = GP%irank
    call CPU_TIME(CPUTime1)

    if(irank.lt.LP(iLayer)%nsize) then

    nstartx = MAX(LP(iLayer)%PartitionInfo(irank+1,1)-GP%nRowBoundary,1)
    nendx   = MIN(LP(iLayer)%PartitionInfo(irank+1,2)+GP%nRowBoundary,LP(iLayer)%NX)
    nstarty = MAX(LP(iLayer)%PartitionInfo(irank+1,3)-GP%nRowBoundary,1)
    nendy   = MIN(LP(iLayer)%PartitionInfo(irank+1,4)+GP%nRowBoundary,LP(iLayer)%NY)


    !/// estimate flow depth for flux component M ///!
    do j = nstarty,nendy
    do i = nstartx,nendx-1
        z1 = LP(iLayer)%Z(i,j);   h1 = LP(iLayer)%H(2,i,j)
        z2 = LP(iLayer)%Z(i+1,j); h2 = LP(iLayer)%H(2,i+1,j)
        m0 = LP(iLayer)%M(1,i,j)
        call checkFluxDirection(GP%PermanentDryLimit,z1,z2,h1,h2,s)
        if(s.eq.999) then
            LP(iLayer)%D_M(i,j) = 0.0d0
        elseif(s.eq.0) then
            if(m0.ge.0.0) then
                LP(iLayer)%D_M(i,j) = (z1+z2)/2.0 + h1
            else
                LP(iLayer)%D_M(i,j) = (z1+z2)/2.0 + h2
            endif
        else
            LP(iLayer)%D_M(i,j) = (z1+z2)/2.0 + MAX(h1,h2)
        endif
        LP(iLayer)%Sign_M(i,j) = s
    enddo
    enddo

    !/// estimate flow depth for flux component N ///!
    do j = nstarty, nendy-1
    do i = nstartx, nendx
        z1 = LP(iLayer)%Z(i,j);   h1 = LP(iLayer)%H(2,i,j)
        z2 = LP(iLayer)%Z(i,j+1); h2 = LP(iLayer)%H(2,i,j+1)
        n0 = LP(iLayer)%N(1,i,j)
        call checkFluxDirection(GP%PermanentDryLimit,z1,z2,h1,h2,s)
        if(s.eq.999) then
            LP(iLayer)%D_N(i,j) = 0.0d0
        elseif(s.eq.0) then
            if(n0.ge.0.0) then
                LP(iLayer)%D_N(i,j) = (z1+z2)/2.0 + h1
            else
                LP(iLayer)%D_N(i,j) = (z1+z2)/2.0 + h2
            endif
        else
            LP(iLayer)%D_N(i,j) = (z1+z2)/2.0 + MAX(h1,h2)
        endif
        LP(iLayer)%Sign_N(i,j) = s
    enddo
    enddo

    endif !if irank is used
    call CPU_TIME(CPUTime2); GP%CPUTime(irank+1,3) = GP%CPUTime(irank+1,3)+CPUTime2-CPUTime1 

    end subroutine reconstructFlowDepth



    subroutine checkFluxDirection(dryLimit,z1,z2,h1,h2,FluxSign)
    ! This subroutine checks the reasonble situations of flux M/N
    ! FluxSign can be {-2,-1,0,1,2,999}, with the same definition as Sign_M/N in LayerParameters
    ! dryLimit      : permanent dry limit defined in GlobalParametes
    ! z1,z2,h1,h2   : water depth and elevation on both sides of flux M/N
    ! 0 :water can flow in both directions; 999:water cannot flow
    ! 1 :water can only flow in positive direction, and from higher to lower
    ! 2 :water can only flow in positive direction, and from lower to higher
    !-1 :water can only flow in negative direction, and from higher to lower
    !-2 :water can only flow in negative direction, and from lower to higher
    ! If water can flow in only one direction, values of h1 and h2 are modified for momentum equation
            
    implicit NONE
    real*8, intent(in)      ::  dryLimit, z1, z2
    real*8, intent(inout)   ::  h1, h2          
    integer*4, intent(out)  ::  FluxSign
        
    if(z1.le.-dryLimit.or.z2.le.-dryLimit) then
        FluxSign = 999; return
    endif
        
    if(z1+h1.le.0.0.or.z2+h2.le.0.0) then
        if(z1+h1.le.0.0.and.z2+h2.le.0.0) then
            FluxSign = 999; return
        elseif(z1.gt.z2.and.z1+h1.gt.0.0.and.z2+h1.le.0.0) then
            FluxSign = 999; return
        elseif(z2.gt.z1.and.z2+h2.gt.0.0.and.z1+h2.le.0.0) then
            FluxSign = 999; return
        endif
    endif
        
    if(z1.le.z2.and.z1+h1.gt.0.0.and.z1+h2.le.0.0) then
        FluxSign = 1; h2 = -z1
    elseif(z1.ge.z2.and.z2+h2.le.0.0.and.z2+h1.gt.0.0) then
        FluxSign = 2; h2 = -z2
    elseif(z2.le.z1.and.z2+h2.gt.0.0.and.z2+h1.le.0.0) then
        FluxSign = -1; h1 = -z2
    elseif(z2.ge.z1.and.z1+h1.le.0.0.and.z1+h2.gt.0.0) then
        FluxSign = -2; h1 = -z1
    else
        FluxSign = 0
    endif
        
    end subroutine checkFluxDirection
